@@ -4,9 +4,11 @@ import (
 	"encoding/base64"
 	"encoding/json"
 	"github.com/cnsilvan/UnblockNeteaseMusic/provider/base"
+	"html"
 	"log"
 	"net/http"
 	"net/url"
+	"path/filepath"
 	"regexp"
 	"strings"
 
@@ -17,6 +19,10 @@ import (
 
 type KuWo struct{}
 
+var blockSongUrl = map[string]json.Number{
+	"2914632520.mp3": "7",
+}
+
 func (m *KuWo) SearchSong(song common.SearchSong) (songs []*common.Song) {
 	song = base.PreSearchSong(song)
 	token := getToken(song.Keyword)
@@ -41,22 +47,22 @@ func (m *KuWo) SearchSong(song common.SearchSong) (songs []*common.Song) {
 				maxIndex = 10
 			}
 			for index, matched := range list {
-				if index >= 1 { //kuwo list order by score default
+				if index >= maxIndex { //kuwo list order by score default
 					break
 				}
 				kuWoSong, ok := matched.(common.MapType)
 				if ok {
 					rid, ok := kuWoSong["rid"].(json.Number)
-					rids:=""
+					rids := ""
 					if !ok {
 						rids, ok = kuWoSong["rid"].(string)
-					}else{
-						rids=rid.String()
+					} else {
+						rids = rid.String()
 					}
 					if ok {
 						songResult := &common.Song{}
-						singerName, _ := kuWoSong["artist"].(string)
-						songName, _ := kuWoSong["name"].(string)
+						singerName := html.UnescapeString(kuWoSong["artist"].(string))
+						songName := html.UnescapeString(kuWoSong["name"].(string))
 						//musicSlice := strings.Split(musicrid, "_")
 						//musicId := musicSlice[len(musicSlice)-1]
 						songResult.PlatformUniqueKey = kuWoSong
@@ -70,7 +76,7 @@ func (m *KuWo) SearchSong(song common.SearchSong) (songs []*common.Song) {
 						}
 						songResult.Name = songName
 						songResult.Artist = singerName
-						songResult.AlbumName, _ = kuWoSong["album"].(string)
+						songResult.AlbumName = html.UnescapeString(kuWoSong["album"].(string))
 						songResult.Artist = strings.ReplaceAll(singerName, " ", "")
 						songResult.MatchScore, ok = base.CalScore(song, songName, singerName, index, maxIndex)
 						if !ok {
@@ -128,6 +134,10 @@ func (m *KuWo) GetSongUrl(searchSong common.SearchMusic, song *common.Song) *com
 					address := string(body)
 					params := reg.FindStringSubmatch(address)
 					if len(params) > 0 {
+						if duration, ok := blockSongUrl[filepath.Base(params[0])]; ok && song.PlatformUniqueKey["duration"].(json.Number) == duration {
+							log.Println(song.PlatformUniqueKey["UnKeyWord"].(string) + "，该歌曲酷我版权保护")
+							return song
+						}
 						song.Url = params[0]
 						return song
 					}
